@@ -117,10 +117,9 @@ bit-endianness, which may be 'big', 'little'.");
 
 /* ------------------------------- count_n ----------------------------- */
 
-/* Return smallest index i for which a.count(vi, 0, i) == n.
-   When n exceeds the total count, the result is a negative
-   number; the negative of the total count + 1, which is useful
-   for displaying error messages. */
+/* Return smallest index i for which a.count(vi, 0, i) == n.  When n exceeds
+   the total count, the result is a negative number; the negative of the
+   total count + 1, which is useful for displaying error messages. */
 static Py_ssize_t
 count_n_core(bitarrayobject *a, Py_ssize_t n, int vi)
 {
@@ -186,10 +185,9 @@ count_n(PyObject *module, PyObject *args)
                             "size (len(a) = %zd)", n, a->nbits);
 
     i = count_n_core(a, n, vi);        /* do actual work here */
-
     if (i < 0)
         return PyErr_Format(PyExc_ValueError, "n = %zd exceeds total count "
-                            "(a.count(%d) = %zd)", n, vi, -i - 1);
+                            "(a.count(%d) = %zd)", n, vi, -(i + 1));
 
     return PyLong_FromSsize_t(i);
 }
@@ -215,12 +213,10 @@ parity(PyObject *module, PyObject *obj)
     a = (bitarrayobject *) obj;
     wbuff = WBUFF(a);
     x = zlw(a);
-    for (i = 0; i < a->nbits / 64; i++)
+    i = a->nbits / 64;
+    while (i--)
         x ^= *wbuff++;
-    for (i = 32; i > 0; i /= 2)
-        x ^= x >> i;
-    x &= 1;
-    return PyLong_FromLong((long) x);
+    return PyLong_FromLong(parity_64(x));
 }
 
 PyDoc_STRVAR(parity_doc,
@@ -1094,7 +1090,7 @@ sc_count(bitarrayobject *a, Py_ssize_t *rts, Py_ssize_t offset, int n)
            nbits = Py_MIN(8 * BSI(n), a->nbits - 8 * offset);
 
        However, on 32-bit machines this will fail for n=4 because 8 * BSI(4)
-       is 1 << 32.  This is problematic, as 32-bit machines can address
+       equals 1 << 32.  This is problematic, as 32-bit machines can address
        at least partially filled type 4 blocks).  Therefore, we first
        limit BSI(n) by the buffer size before multiplying 8. */
     nbits = Py_MIN(8 * Py_MIN(BSI(n), Py_SIZE(a)), a->nbits - 8 * offset);
@@ -1152,7 +1148,7 @@ sc_write_indices(char *str, bitarrayobject *a, Py_ssize_t *rts,
 {
     const char *str_stop = str + n * k;  /* stop position in buffer 'str' */
     const char *buff = a->ob_item + offset;
-    Py_ssize_t m, i;
+    Py_ssize_t m;
 
     assert(1 <= n && n <= 4);
     assert(0 < k && k < 256);  /* note that k cannot be 0 in this function */
@@ -1162,6 +1158,7 @@ sc_write_indices(char *str, bitarrayobject *a, Py_ssize_t *rts,
     rts += offset / SEGSIZE;   /* rts index relative to offset now */
 
     for (m = 0;;) {  /* loop segments */
+        Py_ssize_t i;
         int j, ni;
 
         assert(m + offset / SEGSIZE < NSEG(a->nbits));
@@ -1171,10 +1168,11 @@ sc_write_indices(char *str, bitarrayobject *a, Py_ssize_t *rts,
 
         for (i = m * SEGSIZE;; i++) {  /* loop bytes in segment */
             assert(i < (m + 1) * SEGSIZE && i + offset < Py_SIZE(a));
-            if (buff[i] == 0)
+            if (buff[i] == 0x00)
                 continue;
 
-            for (j = 0; j < 8; j++)  /* loop bits */
+            for (j = 0; j < 8; j++) {  /* loop bits */
+                assert(8 * (offset + i) + j < a->nbits);
                 if (buff[i] & BITMASK(a, j)) {
                     write_n(str, n, 8 * i + j);
                     str += n;
@@ -1185,6 +1183,7 @@ sc_write_indices(char *str, bitarrayobject *a, Py_ssize_t *rts,
                         goto next_segment;
                     }
                 }
+            }
         }
     next_segment:
         m++;
@@ -1235,7 +1234,7 @@ sc_write_sparse(char *str, bitarrayobject *a, Py_ssize_t *rts,
 
    - Now we decide which sparse block type to use.  We do this by
      first calculating the population count for the bitarray buffer size of
-     the NEXT block type.  If the this count is larger than 255 (too large
+     the *next* block type.  If the this count is larger than 255 (too large
      for the count byte) we have to stick with the current type.
      Otherwise we compare the encoded sizes of (a) sticking with the current
      type n, and (b) moving to the next type n+1.  These sizes are calculated
@@ -1243,16 +1242,14 @@ sc_write_sparse(char *str, bitarrayobject *a, Py_ssize_t *rts,
 
      (a) Although we consider sticking with the current type n, we are
          looking at the population for the next type block size.  We
-         have to calculate the encoded size of ALL the type n blocks
-         which would otherwise just be a single type n+1 block.  This
-         size is:
+         calculate the encoded size of *all* the type n blocks which wold
+         otherwise just be a single type n+1 block:
 
-             header size  *  number of blocks  +  n  *  population
+             header size  *  number of blocks   +   n  *  population
 
-     (b) Calculating the encoded size of a single block of type n+1 is
-         much easier:
+     (b) The encoded size of a single block of type n+1 is:
 
-             header size  +  (n + 1)  *  population
+             header size   +   (n + 1)  *  population
 
      As we only need to know which of these sizes is bigger, we can
      substract n * population from both sizes.
@@ -1275,20 +1272,22 @@ sc_encode_block(char *str, Py_ssize_t *len,
     }
 
     for (n = 1; n < 4; n++) {
-        Py_ssize_t next_count, size_a, size_b;
+        Py_ssize_t next_count, nblocks, cost_a, cost_b;
 
         /* population for next block type n+1 */
         next_count = sc_count(a, rts, offset, n + 1);
-        if (next_count >= 256)
+        if (next_count > 255)
             /* too many index bytes for next block type n+1 */
             break;
 
-        /* encoded size of (up to 256) blocks of type n */
-        size_a = (n == 1 ? 1 : 2) * Py_MIN(256, (nbytes - 1) / BSI(n) + 1);
-        /* encoded size of (a single) block of type n+1 */
-        size_b = 2 + next_count;
+        /* number of blocks of type n (up to 256) */
+        nblocks = Py_MIN(256, (nbytes - 1) / BSI(n) + 1);
+        /* cost of nblocks blocks of type n */
+        cost_a = (n == 1 ? 1 : 2) * nblocks;
+        /* cost of a single block of type n+1 */
+        cost_b = 2 + next_count;
 
-        if (size_a <= size_b)
+        if (cost_a <= cost_b)
             /* next block type n+1 is not smaller - use block type n */
             break;
 
